--[[
	Kui Mount
	Kesava-Auchindoun
]]

local strfind, strlower
	= strfind, strlower
local professions

-- Add mounts you don't want to be used here.
-- it must be in the same way as the Celestial Steed is listed by default.
-- i.e. copy and paste the Celestial Steed line, and ONLY change the name.
-- (and it's case sensitive!)
local blacklist = {
	['Celestial Steed'] = true,

}

local function MeetsProfessionRequirement(description)
	-- detect profession requirements
	-- even though IsUsableSpell returned true. siiiiigh
	local usable = true
	local pdesc = select(3, strfind(description, '(requires.-skill.-%.)'))
	
	if pdesc then
		-- find the profession skill level required
		local skill = tonumber(select(3, strfind(pdesc, '(%d+)')))
		local prof
		
		-- and, more annoyingly, name
		if strfind(pdesc, 'engineering') then
			prof = 'engineering'
		elseif strfind(pdesc, 'tailoring') then
			prof = 'tailoring'
		end
		
		if not professions then
			professions = { GetProfessions() }
			
			for x = 1,2 do
				if professions[x] then
					local pname, _, pskill = GetProfessionInfo(professions[x])
					professions[strlower(pname)] = pskill
				end
			end
		end
		
		-- and finally test if we can actually use the mount
		if not professions[prof] or professions[prof] < skill then
			usable = false
		end
	end
	
	return usable
end

local function Mount()
	-- Dismount ----------------------------------------------------------------
	if UnitInVehicle('player') then 
		VehicleExit()
		return
	end
	if IsMounted('player') then
		Dismount()
		return
	end

	if InCombatLockdown() then
		UIErrorsFrame:AddMessage('You are in combat', 1,0,0)
		return
	end
		
	-- Collect usable mounts ---------------------------------------------------
	local mounts = GetNumCompanions('mount')
	if mounts <= 0 then
		UIErrorsFrame:AddMessage('You don\'t have any mounts.', 1,0,0)
		return
	end
	
	local IsAltKeyDown, IsControlKeyDown, IsShiftKeyDown, IsFlyableArea
		= IsAltKeyDown(), IsControlKeyDown(), IsShiftKeyDown(), IsFlyableArea()
	local usable, useFlying, isSwimZone =
		{}, not IsControlKeyDown and IsFlyableArea
	
	-- search professions once per call
	professions = nil
	
	if IsSwimming() then
		local swimZones = {
			['Vashj\'ir'] = true,
			['Abyssal Depths'] = true,
			['Kelp\'thar Forest'] = true,
			['Shimmering Expanse'] = true,
			['Damplight Chamber'] = true
		}
		isSwimZone = swimZones[GetZoneText()]
	end
	
	for i = 1, mounts do
		local _, name, id = GetCompanionInfo('mount', i) 
		local desc = strlower(GetSpellDescription(id))
		
		if (not blacklist[name] or IsAltKeyDown) and
		   IsUsableSpell(id) and
		   MeetsProfessionRequirement(desc)
		then
			local hybrid, flying, profession, isNotUsable
			
			-- detect hybrid/flying mounts
			hybrid = strfind(desc, 'mount changes')
			
			if not hybrid then
				flying = strfind(desc, 'flying')
			end
			
			-- select the seapony in vashj'ir
			if isSwimZone and name == 'Abyssal Seahorse' and not IsShiftKeyDown then
				usable = { id }
				break
			end
		
			if not isNotUsable then
				if useFlying then
					if flying or hybrid then
						tinsert(usable, id)
					end
				else
					if not flying or hybrid then
						tinsert(usable, id)
					end
				end
			end
		end
	end
	
	-- Select usable mount -----------------------------------------------------
	if #usable > 0 then
		local name = GetSpellInfo(usable[math.random(1, #usable)])
		CastSpellByName(name)
	else
		UIErrorsFrame:AddMessage('Couldn\'t find a usable mount.', 1,0,0)
	end
end

--------------------------------------------------------------- Slash command --
SLASH_KUIMOUNT1 = '/kuimount'
SLASH_KUIMOUNT2 = '/mount'

function SlashCmdList.KUIMOUNT()
	Mount()
end